<?

 /*************************************************************************************
 * Filename: backend.php
 *
 * This file is part of the DocIT JAS plugin.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The DocIT JAS plugin is free software under the terms of the
 * GNU General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the plugin or http://jas.junerules.com/plugins/docit/
 *************************************************************************************/



 require_once($_path["include"]."fs.php");
 require_once($_path["include"]."data.php");
 require_once($_path["include"]."toolkit.php");

 require_once($_jasPath["project"]["include"]."backend/data.php");

 class Plugin_Backend_docit extends Base_Plugin_Backend
 {
  function main()
  {
   global $_path;

   $this->fs=new FS();

   $this->tk=new Toolkit();
   unset($this->tk->e);
   $this->tk->e=&$this->e;
   $this->tk->setConfirmOptions("Confirm", "Yes", "enable", "No", "disable");

   //validate input
   if(!($this->topic=request("topic"))||!preg_match("/^\d+$/", $this->topic))
    $this->topic=false;

   if(!($this->directories=$this->fs->directories(($_path["root"]?$_path["root"]:"."), true, true)))
   {
    $this->this->e->add($this->fs->e);
    $this->fs->e->clear();
    return false;
   }

   if(!($this->directory=request("directory"))||!in_array($this->directory, $this->directories))
    $this->directory=false;

   $this->file=false;
   $this->files=Array();
   $this->filesList=false;
   if($this->directory)
   {
    $this->filesList=$this->fs->files($_path["root"].$this->directory, true, true, false, "\.php$");

    if(($this->tmp=request("files"))&&is_array($this->tmp))
     foreach($this->tmp as $this->file)
      if(in_array($this->file, $this->filesList))
       $this->files[]=$this->file;

    if(!($this->file=request("file"))||!in_array($this->file, $this->filesList))
     $this->file=false;
   }

   if(request("s")&&$this->topic&&$this->directory)
   {
    if(!$this->files)
     $this->e->add("No files selected for import", "error");
    else
    {
     if($this->import($this->topic, $this->directory, $this->files))
     {
      $this->topic=false;
      $this->directory=false;
      $this->files=false;
     }
    }
   }

   return true;
  }


  function output()
  {
   global $_path;

   //main section
   $r="<div class=\"twothird\">\n";

   if($this->topic)
   {
    $d=Data::cache("docit_topics", "tree", "up");

    $d->clearValues();
    $d->setValue("parent", $this->topic);
    $d->setShowFields("topic");

    if(!$d->get())
     $this->e->add($d->e);
    else
    {
     $crumbs=Array();
     while($d->next())
      $crumbs[]=$this->tk->link($d->v["topic"], Array(false, Array("directory" => $this->directory, "topic" => $d->v["nr"])));

     if($crumbs)
      $r.="<h2>".$this->tk->icon("log", "medium")." ".implode(" / ", $crumbs)."</h2>\n";
    }
   }

   if($this->directory)
   {
    $current="";
    $crumbs=Array();
    foreach(explode("/", trim($this->directory, "/")) as $dir)
    {
     $current=$current.$dir."/";
     $crumbs[]=$this->tk->link($dir, Array(false, Array("directory" => $current, "topic" => $this->topic)));
    }

    $this->tk->tableRow("files", true);
    $this->tk->tableColumn("files", "", false, false, Array("class" => "checkall"));
    $this->tk->tableColumn("files", $this->tk->icon("cache", "small")." ".implode(" / ", $crumbs), false, false, Array("colspan" => 2));
    $this->tk->tableColumn("files", $this->e->l->t("Classes"));
    $this->tk->tableColumn("files", $this->e->l->t("Functions"));
    $this->tk->tableColumn("files", $this->e->l->t("Variables"));
    $this->tk->tableColumn("files", $this->e->l->t("Lines"));

    if(!$this->filesList)
    {
     $this->tk->tableRow("files");
     $this->tk->tableColumn("files", $this->e->l->t("No relevant files found in directory"), false, false, Array("colspan" => 5));
    }
    else
    {
     foreach($this->filesList as $file)
     {
      $this->tk->tableRow("files");

      if(!($stats=$this->stats($_path["root"].$this->directory.$file))||(!$stats["classes"]&&!$stats["functions"]))
      {
       $this->tk->tableColumn("files", "");
       $this->tk->tableColumn("files", $file, false, false, Array("colspan" => 2));
       $this->tk->tableColumn("files", "", false, false, Array("colspan" => 4));
      }
      else
      {
       $this->tk->tableColumn("files", $this->tk->f->checkbox((in_array($file, $this->files)), false, "files[]", $file));
       $this->tk->tableColumn("files", $this->tk->link($file, Array(false, Array("directory" => $current, "topic" => $this->topic, "file" => $file))), false, false, Array("colspan" => 2));
       $this->tk->tableColumn("files", $stats["classes"], false, false, Array("class" => "nr"));
       $this->tk->tableColumn("files", $stats["functions"], false, false, Array("class" => "nr"));
       $this->tk->tableColumn("files", $stats["variables"], false, false, Array("class" => "nr"));
       $this->tk->tableColumn("files", $stats["lines"], false, false, Array("class" => "nr"));
      }

      if($file==$this->file)
      {
       if($analyzed=$this->analyze($_path["root"].$this->directory.$file))
       {
        $class=false;
        foreach($analyzed["functions"] as $function)
        {
         if($class!=$function["class"])
         {
          $class=$function["class"];

          $this->tk->tableRow("files", true);
          $this->tk->tableColumn("files", "");
          $this->tk->tableColumn("files", $function["class"], false, false, Array("hilite"));
          $this->tk->tableColumn("files", $this->e->l->t("Arguments"));
          $this->tk->tableColumn("files", $this->e->l->t("Returns"), false, false, Array("colspan" => 4));
         }

         $arguments=Array();
         foreach($function["arguments"] as $argument)
          $arguments[]=($argument["reference"]?"&":"")."<b>".$argument["name"]."</b>".($argument["default"]?" = ".$argument["default"]:"");

         $this->tk->tableRow("files");
         $this->tk->tableColumn("files", "");
         $this->tk->tableColumn("files", $this->tk->icon("execute", "small")." ".$function["name"], false, false, Array("hilite"));
         $this->tk->tableColumn("files", implode("<br/>", $arguments));
         $this->tk->tableColumn("files", ($function["return"]?ucfirst($function["return"]):""), false, false, Array("colspan" => 4));
        }
       }
      }
     }
    }

    $r.=$this->tk->table("files");
   }
   else
    $r.="&nbsp; ";

   $r.="</div>\n";


   //form
   $d=Data::cache("docit_topics", "tree", "down");

   $d->clearValues();
   $d->setValue("parent", 0);
   $d->setShowFields("topic", "treedepth");

   $r.="<div class=\"third\">\n";
   $r.=$this->tk->field($this->e->l->t("Topic"), $this->tk->f->select(false, "topic", $d, $this->topic, false, false, "onchange=\"this.form.submit()\""));
   $r.=$this->tk->field($this->e->l->t("Directory"), $this->tk->f->select(false, "directory", $this->directories, $this->directory, false, false, "onchange=\"this.form.submit()\""));

   if($this->topic&&$this->directory&&$this->filesList)
    $r.=$this->tk->confirm();
   else
    $r.=$this->tk->field(false, $this->tk->f->button("confirm", $this->tk->icon("import", "small")." ".$this->e->l->t("Import"), false, false, "submit"));

   $r.="</div>\n";

   //make full page part of the form
   $r=$this->tk->form($r);

   if($this->e->any())
    $r=$this->e->show().$r;

   return $r;
  }


  /*
   * Code analysis
   */

  function import($topic, $directory, $files)
  {
   global $_path;

   //gather existing topics under select topic
   $d=Data::cache("docit_topics", "tree", "down", Array("format" => "sql"));
   $d->setValue("parent", $topic);
   $d->setShowFields("topic", "content", "name", "path", "parent", "catnr");

   if(!$d->get())
   {
    $this->e->add($d->e);
    return false;
   }

   //analyze all files
   $analyzed=Array();
   foreach($files as $file)
    if(!($analyzed[$directory.$file]=$this->analyze($_path["root"].$directory.$file)))
     return false;

   $stats=Array(
    "classes" => Array("added" => 0, "updated" => 0),
    "functions" => Array("added" => 0, "updated" => 0),
   );

   $classes=Array();
   $catnr=$d->getValue("catnr");
   $da=Data::cache("docit_topics", "add", "item");
   $de=Data::cache("docit_topics", "edit", "item");

   //add/update classes
   foreach($analyzed as $file => $v)
   {
    foreach($v["classes"] as $class)
    {
     $d->reset();
     $found=false;
     $content=false;
     while($d->search("topic", $class["name"]))
     {
      if($d->v["parent"]==$topic)
      {
       $found=$d->v["nr"];
       $content=$d->v["content"];
       break;
      }
     }

     $oldContent=$content;

     if(!$content||!($lb=$this->tk->m->detectLineBreaks($content)))
      $lb="\n";


     //class section
     if($content&&preg_match("!\[c=\"[^\"]*".$class["name"]."[^\"]*\"([^\]]*?) l=Class([^\]]*)](.*?)\[c\]!sm", $content, $m)) //class section included
     {
      $section="[c=\"".$class["name"].($class["extends"]?" extends ".$class["extends"]:"")."\"".$m[1]." l=Class".$m[2]."]";

      if(!strstr($m[3], $file)) //filename not in
       $section.=$file.($m[3]?$lb.$lb:"");

      $section.=$m[3]."[c]";
      $content=str_replace($m[0], $section, $content);
     }
     else //no class section in text
     {
      //add class section above text
      $section="[c=\"".$class["name"].($class["extends"]?" extends ".$class["extends"]:"")."\" l=Class]".$file."[c]".$lb.$lb;

      if($content)
       $content=$section.$content;
      else
       $content=$section."Description...";
     }


     //variables section
     if($content&&preg_match_all("!\[c=\"([^\"]+?)( \((\w+)\))?\"([^\]]*?) l=Variable([^\]]*)](.*?)\[c\][^\[]*!sm", $content, $m)) //variable section(s) included
     {
      $c=0;
      $section="";
      foreach($v["variables"] as $variable)
      {
       if($class["name"]==$variable["class"])
       {
        if(isset($m[1][$c])) //topic contains a variable at same place
        {
         if($m[1][$c]==$variable["name"]) //same name
          $section.="[c=\"".$variable["name"].($m[3][$c]?" (".$m[3][$c].")":"")."\"".$m[4][$c]." l=Variable".$m[5][$c]."]".$m[6][$c]."[c]".$lb; //keep current entry (tho type may be overwritten)
         else if($m[3][$c]==$variable["type"]) //same type
          $section.="[c=\"".$variable["name"].($variable["type"]?" (".$variable["type"].")":"")."\"".$m[4][$c]." l=Variable".$m[5][$c]."]".$m[6][$c]."[c]".$lb; //rename current entry
         else //different name + type
         {
          $section.="[c=\"".$m[1][$c]." (deprecated)\"".$m[4][$c]." l=Variable".$m[5][$c]."]".$m[6][$c]."[c]".$lb; //add deprecated entry
          $section.="[c=\"".$variable["name"].($variable["type"]?" (".$variable["type"].")":"")."\" l=Variable][c]".$lb; //add new entry
         }
        }
        else //variable doesn't exist in documentation
         $section.="[c=\"".$variable["name"].($variable["type"]?" (".$variable["type"].")":"")."\" l=Variable][c]".$lb; //add new entry

        $c++;
       }
      }

      for($c=$c+1;$c<sizeof($m[1]);$c++) //add remaining entries in topic
       $section.="[c=\"".$m[1][$c]." (deprecated)\"".$m[4][$c]." l=Variable".$m[5][$c]."]".$m[6][$c]."[c]".$lb; //add deprecated entry

      $content=str_replace(implode($m[0]), $section, $content);
     }
     else //no variable section(s) in text
     {
      $section="";
      foreach($v["variables"] as $variable)
       if($class["name"]==$variable["class"])
        $section.="[c=\"".$variable["name"].($variable["type"]?" (".$variable["type"].")":"")."\" l=Variable][c]".$lb;

      if($section)
       $content.=$lb.$lb.$section;
     }


     //add/update topic
     if($found)
     {
      if($content!=$oldContent)
      {
       $de->e->clear();
       $de->clearValues();
       $de->setValue("nr", $found);
       $de->setValue("content", $content);

       if(!$de->submit())
        $this->e->add($de->e);
       else
        $stats["classes"]["updated"]++;
      }

      $classes[$class["name"]]=$found; //store class topic id
     }
     else //not found
     {
      $da->e->clear();
      $da->clearValues();
      $da->setValue("catnr", $catnr);
      $da->setValue("parent", $topic);
      $da->setValue("published", true);

      $da->setValue("topic", $class["name"]);
      $da->setValue("content", $content);

      if(!$da->submit())
       $this->e->add($da->e);
      else
      {
       $stats["classes"]["added"]++;
       $classes[$class["name"]]=$da->getValue("nr"); //store class topic id
      }
     }
    }
   }


   //add/update functions/methods
   foreach($analyzed as $file => $v)
   {
    foreach($v["functions"] as $function)
    {
     $d->reset();
     $found=false;
     $content=false;
     if($function["class"]) //method
     {
      if(!isset($classes[$function["class"]]))
      {
       $this->e->add("Unable to find class topic: \"".$function["class"]."\" for method: \"".$function["name"]."\"", "error");
       continue;
      }

      $class=$classes[$function["class"]];
      while($d->search("topic", $function["name"]))
      {
       if($d->v["parent"]==$class)
       {
        $found=$d->v["nr"];
        $content=$d->v["content"];
        break;
       }
      }
     }
     else //function
     {
      $class=false;
      while($d->search("topic", $function["name"]))
      {
       if($d->v["parent"]==$topic)
       {
        $found=$d->v["nr"];
        $content=$d->v["content"];
        break;
       }
      }
     }

     $oldContent=$content;

     if(!$content||!($lb=$this->tk->m->detectLineBreaks($content)))
      $lb="\n";


     //function section
     if($content&&preg_match("!\[c=\"[^\"]*".($class?$function["class"]." :: ":"").$function["name"]."[^\"]*\"([^\]]*?) l=(Method|Function)([^\]]*)](.*?)\[c\]!sm", $content, $m)) //function section included
     {
      $section="[c=\"".($class?$function["class"]." :: ":"").$function["name"]."\"".$m[1]." l=".($class?"Method":"Function").$m[3]."]";

      if(!strstr($m[4], $file)) //filename not in
       $section.=$file.($m[4]?$lb.$lb:"");

      $section.=$m[4]."[c]";
      $content=str_replace($m[0], $section, $content);
     }
     else //no function section in text
     {
      //add function section above text
      $section="[c=\"".($class?$function["class"]." :: ":"").$function["name"]."\" l=".($class?"Method":"Function")."]".$file."[c]".$lb.$lb;

      if($content)
       $content=$section.$content;
      else
       $content=$section."Description...";
     }


     //arguments section
     if($content&&preg_match_all("!\[c=\"\s*([a-zA-Z0-9_]+)(\s*=\s*([^\"]+?))?\s*(\(reference\)|\(deprecated\))?\"([^\]]*?) l=Argument([^\]]*)](.*?)\[c\][^\[]*!sm", $content, $m)) //argument section(s) included
     {
      $c=0;
      $section="";
      foreach($function["arguments"] as $argument)
      {
       if(isset($m[1][$c])) //topic contains a argument at same place
       {
        if($m[1][$c]==$argument["name"]) //same name
         $section.="[c=\"".$argument["name"].($argument["default"]?" = ".$argument["default"]:"").($argument["reference"]?" (reference)":"")."\"".$m[5][$c]." l=Argument".$m[6][$c]."]".$m[7][$c]."[c]".$lb; //keep current entry (tho type may be overwritten)
        else //different name + type
        {
         $section.="[c=\"".$m[1][$c]." (deprecated)\"".$m[5][$c]." l=Argument".$m[6][$c]."]".$m[7][$c]."[c]".$lb; //add deprecated entry
         $section.="[c=\"".$argument["name"].($argument["default"]?" = ".$argument["default"]:"").($argument["reference"]?" (reference)":"")."\" l=Argument][c]".$lb; //add new entry
        }
       }
       else //argument doesn't exist in documentation
        $section.="[c=\"".$argument["name"].($argument["default"]?" = ".$argument["default"]:"").($argument["reference"]?" (reference)":"")."\" l=Argument][c]".$lb; //add new entry

       $c++;
      }

      for($c=$c+1;$c<sizeof($m[1]);$c++) //add remaining entries in topic
       $section.="[c=\"".$m[1][$c].($m[3][$c]?" = ".$m[3][$c]:"")." (deprecated)\"".$m[5][$c]." l=Argument".$m[6][$c]."]".$m[7][$c]."[c]".$lb; //add deprecated entry

      $content=str_replace(implode($m[0]), $section.$lb, $content);
     }
     else //no argument section(s) in text
     {
      $section="";
      foreach($function["arguments"] as $argument)
       $section.="[c=\"".$argument["name"].($argument["default"]?" = ".$argument["default"]:"").($argument["reference"]?" (reference)":"")."\" l=Argument][c]".$lb;

      if($section)
       $content.=$lb.$lb.$section;
     }


     //return section
     if(!preg_match("!\[c=\"[^\"]+\"([^\]]*?) l=Returns([^\]]*)](.*?)\[c\]!sm", $content, $m)) //return section included
     {
      $section="[c=\"".($function["return"]?ucfirst($function["return"]):"Nothing")."\" l=Returns][c]";
      $content.=$lb.$section;
     }


     //example section
     if(!preg_match("!\[c=\"[^\"]+ example\"([^\]]*?) l=PHP([^\]]*)](.*?)\[c\]!sm", $content, $m)) //no example section included
     {
      $section=false;
      $arguments=Array();
      foreach($function["arguments"] as $argument)
       $arguments[]=($argument["reference"]?"&":"")."\$".$argument["name"].($argument["default"]?" = ".$argument["default"]:"");

      if($class)
      {
       if($function["name"]=="__construct") //constructor
       {
        $section="[c=\"".$function["class"]." class example\" l=PHP]<?
   \$".strtolower($function["class"])."=new [b]".$function["class"]."[b](".implode(", ", $arguments).");  [i]//Init ".$function["class"]." object[i]
?>[c]";
       }
       else if(substr($function["name"], 0, 2)!="__") //normal function (not __construct/__clone/__destruct)
       {
        $section="[c=\"".$function["class"]." ".ucfirst($function["name"])." method example\" l=PHP]<?
  \$".strtolower($function["class"])."->[b]".$function["name"]."[b](".implode(", ", $arguments).");  [i]//Call ".$function["class"]."::".$function["name"]." method[i]
?>[c]";
       }
      }
      else //function
      {
       $section="[c=\"".ucfirst($function["name"])." function example\" l=PHP]<?
  [b]".$function["name"]."[b](".implode(", ", $arguments).");  [i]//Call ".$function["name"]." function[i]
?>[c]";
      }

      if($section)
       $content.=$lb.$lb.$section;
     }


     //add/update topic
     if($found)
     {
      if($content!=$oldContent)
      {
       $de->e->clear();
       $de->clearValues();
       $de->setValue("nr", $found);
       $de->setValue("content", $content);

       if(!$de->submit())
        $this->e->add($de->e);
       else
        $stats["functions"]["updated"]++;
      }
     }
     else //not found
     {
      $da->e->clear();
      $da->clearValues();
      $da->setValue("catnr", $catnr);
      $da->setValue("parent", ($class?$class:$topic));
      $da->setValue("published", true);

      $da->setValue("topic", $function["name"]);
      $da->setValue("content", $content);

      if(!$da->submit())
       $this->e->add($da->e);
      else
       $stats["functions"]["added"]++;
     }
    }
   }


   //report number of changes
   $changes=false;
   foreach($stats as $k => $stat)
   {
    foreach($stat as $action => $v)
    {
     if($v)
     {
      $this->e->add(ucfirst($action)." ".$v." ".$k, "note");
      $changes=true;
     }
    }
   }

   if(!$changes)
    $this->e->add("All topics are up-to-date", "note");

   return true;
  }

  function stats($file)
  {
   if(!($analyzed=$this->analyze($file)))
    return false;

   return Array(
    "lines" => substr_count($analyzed["code"], "\n"),
    "classes" => sizeof($analyzed["classes"]),
    "functions" => sizeof($analyzed["functions"]),
    "variables" => sizeof($analyzed["variables"]),
   );
  }

  function analyze($file)
  {
   if(!$this->fs->exists($file))
   {
    $this->e->add("Analyze code failed, file: \"".$file."\" doesn't exist", "error");
    return false;
   }

   if(!($code=$this->fs->read($file)))
   {
    $this->e->add("Analyze code failed, unable to read code from: \"".$file."\"", "error");
    return false;
   }

   switch(strtolower(preg_replace("/.+\.([^\.]+)$/", "\\1", $file)))
   {
    case "php":
     return $this->analyzePHP($code);

    default: break;
   }

   $this->e->add("Analyze code failed, unknown extension for file: \"".$file."\"", "error");
   return false;
  }

  function analyzePHP($code)
  {
   $results=Array(
    "code" => $code,
    "classes" => Array(),
    "functions" => Array(),
    "variables" => Array(),
   );

   while($code)
   {
    if(!preg_match("/^.*?\n\s*(class|function|return|\\\$this)/smi", $code, $m))
     break;

    $code=substr($code, strlen($m[0])); //remove parsed section from code

    switch(strtolower($m[1]))
    {
     case "class":
     {
      if(preg_match("/^\s+([a-z0-9_]+)(\s+extends\s+([a-z0-9_]+))?/i", $code, $m))
      {
       $results["classes"][]=Array(
        "name" => $m[1],
        "extends" => (isset($m[3])?$m[3]:false),
       );

       $code=substr($code, strlen($m[0])); //remove parsed section from code
      }

      break;
     }
     case "function":
     {
      if(preg_match("/^\s+([a-z0-9_]+)\s*\(([^\)]*)\)/i", $code, $m))
      {
       $arguments=Array();
       foreach(explode(",", $m[2]) as $argument)
       {
        if(preg_match("/\s*(&?)\s*\\$([^=\s]+)(=(.+))?/", $argument, $m2))
        {
         $arguments[]=Array(
          "name" => $m2[2],
          "default" => (isset($m2[4])?trim($m2[4], " \t\n\r\""):false),
          "reference" => ($m2[1]?true:false),
         );
        }
       }

       if($results["classes"])
        $class=$results["classes"][sizeof($results["classes"])-1]["name"];
       else
        $class=false;

       if(substr($m[1], 0, 2)=="__") //__construct/__clone/__destruct
        $return="instance";
       else
        $return=false;

       $results["functions"][]=Array(
        "class" => $class,
        "name" => $m[1],
        "arguments" => $arguments,
        "return" => $return,
       );

       $code=substr($code, strlen($m[0])); //remove parsed section from code
      }

      break;
     }
     case "\$this":
     {
      if(preg_match("/^->([a-z0-9_]+)\s*=/i", $code, $m))
      {
       if($results["classes"])
       {
        $class=$results["classes"][sizeof($results["classes"])-1]["name"];

        $found=false;
        foreach($results["variables"] as $variable)
        {
         if($variable["class"]==$class&&$variable["name"]==$m[1])
         {
          $found=true;
          break;
         }
        }

        if(!$found)
        {
         $code=substr($code, strlen($m[0])); //remove parsed section from code

         if(preg_match("/^\s*(NULL\s*)?;/", $code))
          $type="NULL";
         else if(preg_match("/^\s*(true|false)\s*;/", $code))
          $type="boolean";
         else if(preg_match("/^\s*\d+\s*;/", $code))
          $type="integer";
         else if(preg_match("/^\s*[\d\.]+\s*;/", $code))
          $type="float";
         else if(preg_match("/^\s*\"[^\"]*\"\s*;/", $code))
          $type="text";
         else if(preg_match("/^\s*new /i", $code))
          $type="object";
         else
          $type=false;

         $results["variables"][]=Array(
          "class" => $class,
          "name" => $m[1],
          "type" => $type,
         );
        }
       }
      }

      break;
     }
     case "return":
     {
      if(!$results["functions"]) //no previous function
       break;

      $previous=sizeof($results["functions"])-1;
      if($results["functions"][$previous]["return"]=="mixed"||$results["functions"][$previous]["return"]=="instance") //don't bother
       break;
      
      if(preg_match("/^\s*(NULL\s*)?;/", $code)) //return;
       $type="NULL";
      else if(preg_match("/^\s*(true|false)\s*;/", $code)) //return true;
       $type="boolean";
      else if(preg_match("/^\s*\d+\s*;/", $code)) //return 123;
       $type="integer";
      else if(preg_match("/^\s*[\d\.]+\s*;/", $code)) //return 1.23;
       $type="float";
      else if(preg_match("/^\s*\"[^\"]*\"\s*;/", $code)) //return "text";
       $type="text";
      else if(preg_match("/^\s*\\$[a-z0-9_\->]+\s*;/i", $code)) //return $variable;
       $type="variable";
      else if(preg_match("/^\s*\\$[a-z0-9_\->]+\([^\)]*\)\s*;/i", $code)) //return $object->method;
       $type="function";
      else
       $type=false;

      if($results["functions"][$previous]["return"]===false) //no return value set
       $results["functions"][$previous]["return"]=$type;
      else if($results["functions"][$previous]["return"]!=$type) //multiple return value types
       $results["functions"][$previous]["return"]="mixed";

      $code=substr($code, strpos($code, ";")); //remove parsed section from code
      break;
     }

     default: break;
    }
   }

   return $results;
  }
 }
?>
