<?

 /*************************************************************************************
 * Filename: docit.php
 *
 * This file is part of the DocIT JAS plugin.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The DocIT JAS plugin is free software under the terms of the
 * GNU General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the plugin or http://jas.junerules.com/plugins/docit/
 *************************************************************************************/


 require_once($_path["include"]."data.php");
 require_once($_path["format"]."template.php");
 require_once($_path["plugins"]."lister/lister.php");

 class Plugin_docit extends Plugin_lister
 {
  function main()
  {
   global $_benchmark, $_docitCommentSubmitted, $_config;

   if(!$this->config)
    return false;

   $this->commentError=false;

   //only submit in topic/full mode to ensure error is set to correct template
   if($this->config["listtype"]!="Topic"&&$this->config["listtype"]!="Tree")
    return true;

   //ensure comments are only submitted once
   if($_docitCommentSubmitted)
    return true;

   //comment submit
   $_docitCommentSubmitted=true;
   if($this->commentText=request("comment"))
   {
    if($_benchmark) $_benchmark->start("Submit comment");

    $d=Data::cache("docit_comments", "add", "item");

    if($_config->userAnonymous())
    {
     $d->addField("captcha", "captcha", "form", "", 0, 5, "", "Captcha");
     $d->setRequestValue("captcha");
    }

    $d->setValue("text", $this->commentText);
    $this->commentEntry=$d->getValue("docittopic");

    if(!$d->submit())
     $this->commentError=$d->e->show();

    if($_benchmark) $_benchmark->stop("Submit comment");
   }

   return true;
  }

  function _getData()
  {
   global $_benchmark;

   if(!$this->config)
    return false;

   if($_benchmark) $_benchmark->start("Data object");

   if($this->config["listtype"]=="Topic")
   {
    $d=Data::cache("docit_topics", "view", "all");
    $d->clearValues();

    if($this->config["curlies"]) //convert {} to []
     $d->addMutilation("content", "replace", "/{(.*?)}/", "[\\1]");

    //set show fields so lister->cache takes these in to account when checking cache
    $d->setShowFields("comments");
    $d->getReferenceData("comments")->setShowFields("text");

    $d->setValue("catnr", $this->config["listcatnr"]);

    if(request("search")) //index search
    {
     $d->setAction("search");
     $d->setExtra("index");

     $d->setValue("search", request("search"));
     $d->sortOrder("fulltext-");

     if(!$d->getValue("show_total")) //ensure limit is set
      $d->setValue("show_total", 100);

     if(!$d->get(false, true)||!$d->getValue("show_count")) //no results, try normal search
     {
      $d->setExtra("full");

      $d->unsetValue("fulltext");
      $d->unsetValue("show_count");

      $d->setValue("search", request("search"));
      $d->sortOrder("date-");
     }
     else //results
      $d->setValue("search", request("search")); //re-set search
    }
    else if(request("p", 1))
    {
     $path=request("p");
     array_shift($path);

     $d->setValue("path", implode("/", $path));
    }
    else //first level entry
     $d->setValue("parent", 0);

    //display published or unpublished if user has edit rights
    $d->setValue("r", "00000000001", 2);
    $d->setOperator("r", "&", 2);
    $d->setOperant("r", "OR", 2);
    $d->setValue("published", true, 2);
   }
   else if($this->config["listtype"]=="Tree"||$this->config["listtype"]=="Index")
   {
    $d=Data::cache("docit_topics", "tree", "down");
    $d->clearValues();

    if($this->config["listtype"]=="Tree"&&$this->config["curlies"]) //convert {} to []
     $d->addMutilation("content", "replace", "/{(.*?)}/", "[\\1]");

    //set show fields so lister->cache takes these in to account when checking cache
    $d->setShowFields("comments");
    $d->getReferenceData("comments")->setShowFields("text");

    if($this->config["listdepth"])
     $d->setValue("treedepth", $this->config["listdepth"]-1);

    $d->setValue("catnr", $this->config["listcatnr"]);
    $d->setValue("parent", 0);

    //display published or unpublished if user has edit rights
    $d->setValue("r", "00000000001", 2);
    $d->setOperator("r", "&", 2);
    $d->setOperant("r", "OR", 2);
    $d->setValue("published", true, 2);
   }
   else //Latest comments
   {
    $d=Data::cache("docit_comments", "view", "all");
    $d->sortOrder("updated-");

    $d->clearValues();

    $d->setValue("catnr", $this->config["listcatnr"]);

    $d->setValue("show_total", $this->config["listitems"]);
   }

   if($_benchmark) $_benchmark->stop("Data object");

   return $d;
  }


  function cache(&$cacheTime, &$cacheKey)
  {
   //no caching on comment error or search
   if($this->commentError||request("search"))
    return false;

   return parent::cache($cacheTime, $cacheKey);
  }

  function _cacheTemplateUpdated(&$cacheTime, &$dc)
  {
   global $_path;

   if($this->config["listtype"]=="Topic") //item/list template
   {
    if($this->config["templateitem"])
     $time=FS::getTimeS($_path["project"]["templates"].$this->config["templateitem"], "modified");
    else
     $time=FS::getTimeS($this->plugin->path["templates"].$this->plugin->templateitem, "modified");
   }
   else if($this->config["listtype"]=="Tree") //list template
   {
    if($this->config["templatelist"])
     $time=FS::getTimeS($_path["project"]["templates"].$this->config["templatelist"], "modified");
    else
     $time=FS::getTimeS($this->plugin->path["templates"].$this->plugin->templatelist, "modified");
   }
   else if($this->config["listtype"]=="Index") //index template
   {
    if($this->config["templatelist"])
     $time=FS::getTimeS($_path["project"]["templates"].$this->config["templatelist"], "modified");
    else
     $time=FS::getTimeS($this->plugin->path["templates"].$this->plugin->templateindex, "modified");
   }
   else //Latest comments
   {
    if($this->config["templatelist"])
     $time=FS::getTimeS($_path["project"]["templates"].$this->config["templatelist"], "modified");
    else
     $time=FS::getTimeS($this->plugin->path["templates"].$this->plugin->templatelatestcomments, "modified");
   }

   if($cacheTime<$time)
    $cacheTime=$time;

   return true;
  }


  function output()
  {
   global $_path, $_config, $_benchmark;

   if($_benchmark) $_benchmark->start("Get content");

   if(!($d=$this->_getData()))
    return false;

   $id=$d->getID();

   if($this->config["cmspage"])
   {
    $tmp=Data::cache("pager_categories", "view", "item");

    $tmp->setValue("nr", $this->config["cmspagenr"]);
    $tmp->setShowFields("urlname");

    if($tmp->get()&&$tmp->next())
     $cmspage=$tmp->v["urlname"];
    else
     $cmspage=$this->config["cmspagenr"];

    unset($tmp);
   }

   $d->addField("sort_order", "text", "show", "control", false, false, implode(",", $d->sortOrder));

   if($_config->get("prettyurls")) //mod rewrite
   {
    if($this->config["cmspage"])
     $d->addField("cms_url", "text", "", "control", 0, false, $_path["HTML"]["current"].$cmspage."/");
    else
     $d->addField("cms_url", "text", "", "control", 0);

    $d->addField("base_url", "text", "", "control", 0, false, $_path["HTML"]["current"]);
    $d->addField("root_url", "text", "", "control", 0, false, $_path["root"]);

    $tmp=$_path["HTML"]["current"];
    foreach($this->plugin->vars as $k => $v)
     if($k=="p"||substr($k, 0, 2)=="p[")
      $tmp.=$v."/";

    $d->addField("page_url", "text", "", "control", 0, false, $tmp);

    unset($tmp);
   }
   else
   {
    if($this->config["cmspage"])
     $d->addField("cms_url", "text", "", "control", 0, false, $_path["HTML"]["current"].htmlentities("?=".$cmspage."&"));
    else
     $d->addField("cms_url", "text", "", "control", 0);

    $d->addField("base_url", "text", "", "control", 0, false, "");
    $d->addField("root_url", "text", "", "control", 0, false, $_path["root"]);
    $d->addField("page_url", "text", "", "control", 0, false, htmlentities($this->plugin->url()));
   }

   //get comments category + rights
   if($this->config["listtype"]=="Topic"||$this->config["listtype"]=="Tree")
   {
    $dtmp=Data::cache("categories", "view", "all");
    $dtmp->setShowFields("nr");
    $dtmp->clearValues();

    $dtmp->setValue("catnr", $this->config["listcatnr"]);
    $dtmp->setValue("text", "DOCIT COMMENTS CONTAINER");
    $dtmp->setValue("show_total", 1);

    if(!$dtmp->get()||!$dtmp->next()) //can't get comments category, either no rights or setup needs to be rerun (DOCIT COMMENTS CONTAINER in docit category should exist)
    {
     $d->addField("comment_catnr", "nr", "", "control", 0, false, false);
     $d->addField("comment_rights", "boolean", "", "control", false, false, false);
     $d->addField("comment_captcha", "boolean", "", "control", false, false, false);
    }
    else
    {
     $d->addField("comment_catnr", "nr", "", "control", 0, false, $dtmp->v["nr"]);

     if($dtmp->r("add", "content"))
     {
      $d->addField("comment_rights", "boolean", "", "control", false, false, true);

      if($_config->userAnonymous())
       $d->addField("comment_captcha", "boolean", "", "control", false, false, true);
      else
       $d->addField("comment_captcha", "boolean", "", "control", false, false, false);
     }
     else
     {
      $d->addField("comment_rights", "boolean", "", "control", false, false, false);
      $d->addField("comment_captcha", "boolean", "", "control", false, false, false);
     }
    }
   }

   //comment submit items
   $d->addField("comment_text", "text", "", "control", 0, false, "");
   $d->addField("comment_error", "text", "", "control", 0, false, "");

   //load template
   if($this->config["listtype"]=="Topic") //item template
   {
    //get number of results
    $dtmp=clone $d;
    $dtmp->get(false, true);

    if($dtmp->getValue("show_count")>1) //more than 1 result
    {
     if($this->config["templatelist"])
      $templateFile=$_path["project"]["templates"].$this->config["templatelist"];
     else
      $templateFile=$this->plugin->path["templates"].$this->plugin->templatelist;
    }
    else //1 result
    {
     if($this->config["templateitem"])
      $templateFile=$_path["project"]["templates"].$this->config["templateitem"];
     else
      $templateFile=$this->plugin->path["templates"].$this->plugin->templateitem;
    }

    unset($dtmp);
   }
   else if($this->config["listtype"]=="Tree") //list template
   {
    if($this->config["templatelist"])
     $templateFile=$_path["project"]["templates"].$this->config["templatelist"];
    else
     $templateFile=$this->plugin->path["templates"].$this->plugin->templatelist;
   }
   else if($this->config["listtype"]=="Index") //index template
   {
    if($this->config["templatelist"])
     $templateFile=$_path["project"]["templates"].$this->config["templatelist"];
    else
     $templateFile=$this->plugin->path["templates"].$this->plugin->templateindex;
   }
   else //Latest comments
   {
    if($this->config["templatelist"])
     $templateFile=$_path["project"]["templates"].$this->config["templatelist"];
    else
     $templateFile=$this->plugin->path["templates"].$this->plugin->templatelatestcomments;
   }


   if($_benchmark) $_benchmark->start("Template");

   $template=new Format_Template(true);

   if(!$template->read($templateFile)||!$template->analyze($d))
   {
    $this->e->add($template->e);
    return false;
   }

   if($template->markerExists("_fields")&&$this->config["showfields"]) //_fields marker in template and predefined fields set
    $d->setShowFields(array_merge($this->config["showfields"], $d->check("use", "show"))); //added predefined fields to show fields
   //else //use template defined fields

   if($_benchmark) $_benchmark->stop("Template");

   if($_benchmark) $_benchmark->start("Get data");

   //get data
   $success=$d->get($template->getLevel(), ($this->config["listtype"]=="Topic"||$this->config["listtype"]=="Tree")); //get show_count for Topic/Tree

   if(!$success)
   {
    $this->e->add($d->e);
    return false;
   }

   //set sort_order, page_url, base_url and root_url values
   for($c=0;$c<$d->vmax;$c++)
   {
    if($this->config["listtype"]=="Topic"||$this->config["listtype"]=="Tree")
    {
     $d->setDefaultValue("comment_catnr", $c);
     $d->setDefaultValue("comment_rights", $c);
    }

    if($this->commentError&&$d->getValue("nr", $c)==$this->commentEntry)
    {
     $d->setValue("comment_text", $this->commentText, $c);
     $d->setValue("comment_error", $this->commentError, $c);
    }
    else
    {
     $d->setDefaultValue("comment_text", $c);
     $d->setDefaultValue("comment_error", $c);
    }

    $d->setDefaultValue("sort_order", $c);
    $d->setDefaultValue("page_url", $c);
    $d->setDefaultValue("base_url", $c);
    $d->setDefaultValue("root_url", $c);
    $d->setDefaultValue("cms_url", $c);
   }

   //fix content
   $page_url=$d->getValue("page_url");
   for($c=0;$c<$d->vmax;$c++)
   {
    if($content=$d->getValue("content", $c))
    {
     $content=preg_replace("/(<a href=\")([A-Za-z0-9\_]+[A-Za-z0-9\_\/]+)(\")/", "\\1".$page_url."\\2\\3", $content); //fix relative links (e.g. Chapter1/Chapter2) to page links

     if($this->config["codeheaders"]) //add code headers
     {
      $content=preg_replace("!(<div[^>]+class=\"code\"[^>]*>[^<]*<div[^>]+class=\"title\"[^>]*>[^<]+ extends )([^<]+)(</div><div[^>]+class=\"language\"[^>]*>Class)!", "\\1<a href=\"".$page_url.preg_replace("/\w+$/", "", $d->getValue("path", $c))."\\2\" title=\"\\2\">\\2</a>\\3", $content, 1); //Add extended class link
      $content=preg_replace("!<div[^>]+class=\"code\"[^>]*>[^<]*<div[^>]+class=\"title\"[^>]*>[^<]*</div><div[^>]+class=\"language\"[^>]*>Argument!", "<h2>Arguments</h2>\\0", $content, 1); //Add arguments header
      $content=preg_replace("!<div[^>]+class=\"code\"[^>]*>[^<]*<div[^>]+class=\"title\"[^>]*>[^<]*</div><div[^>]+class=\"language\"[^>]*>Returns!", "<h2>Return values</h2>\\0", $content, 1); //Add return values header
      $content=preg_replace("!<div[^>]+class=\"code\"[^>]*>[^<]*<div[^>]+class=\"title\"[^>]*>[^<]*example</div><div[^>]+class=\"language\"[^>]*>PHP!", "<h2>Examples</h2>\\0", $content, 1); //Add example header
      $content=preg_replace("!<div[^>]+class=\"code\"[^>]*>[^<]*<div[^>]+class=\"title\"[^>]*>[^<]*</div><div[^>]+class=\"language\"[^>]*>Variable!", "<h2>Class variables</h2>\\0", $content, 1); //Add variables header
     }

     $d->setValue("content", $content, $c);
    }
   }

   if($_benchmark) $_benchmark->stop("Get data");
   if($_benchmark) $_benchmark->start("Populate template");

   if($d->e->any("fatal"))
   {
    $this->e->add($d->e);
    return false;
   }

   if(!$template->populate())
   {
    $this->e->add($template->e);
    return false;
   }

   $output=$template->output();

   if($_benchmark) $_benchmark->stop("Populate template");

   return $output;
  }

 }
?>
